@@ -37,6 +37,7 @@
 #include <getopt.h>
 #include <sys/ioctl.h>
 #include <sys/socket.h>
+#include <assert.h>
 
 #include <bluetooth/bluetooth.h>
 #include <bluetooth/hci.h>
@@ -58,6 +59,8 @@
 #include "headset.h"
 #include "glib-helper.h"
 #include "dbus-common.h"
+#include "../src/adapter.h"
+#include "../src/device.h"
 
 #define DC_TIMEOUT 3000
 
@@ -526,6 +529,41 @@ static int sco_connect(struct audio_device *dev, headset_stream_cb_t cb,
 	return 0;
 }
 
+static int hfp_cmp(struct headset *hs)
+{
+	if (hs->hfp_active)
+		return 0;
+	else
+		return -1;
+}
+
+static void send_foreach_headset(GSList *devices,
+					int (*cmp)(struct headset *hs),
+					char *format, ...)
+{
+	GSList *l;
+	va_list ap;
+
+	for (l = devices; l != NULL; l = l->next) {
+		struct audio_device *device = l->data;
+		struct headset *hs = device->headset;
+		int ret;
+
+		assert(hs != NULL);
+
+		if (cmp && cmp(hs) != 0)
+			continue;
+
+		va_start(ap, format);
+		ret = headset_send_valist(hs, format, ap);
+		if (ret < 0)
+			error("Failed to send to headset: %s (%d)",
+					strerror(-ret), -ret);
+		va_end(ap);
+	}
+}
+
+
 static void hfp_slc_complete(struct audio_device *dev)
 {
 	struct headset *hs = dev->headset;
@@ -755,14 +793,15 @@ static int response_and_hold(struct audio_device *device, const char *buf)
 	if (strlen(buf) < 8)
 		return -EINVAL;
 
-	if (buf[7] == '=')
+	if (buf[7] == '=') {
 		telephony_response_and_hold_req(device, atoi(&buf[8]) < 0);
-	else if (ag.rh >= 0)
-		return headset_send(hs, "\r\n+BTRH:%d\r\n", ag.rh);
-	else
-		return headset_send(hs, "\r\nOK\r\n", ag.rh);
+		return 0;
+	}
 
-	return 0;
+	if (ag.rh >= 0)
+		headset_send(hs, "\r\n+BTRH:%d\r\n", ag.rh);
+
+	return headset_send(hs, "\r\nOK\r\n", ag.rh);
 }
 
 int telephony_last_dialed_number_rsp(void *telephony_device, cme_error_t err)
@@ -784,7 +823,20 @@ int telephony_dial_number_rsp(void *telephony_device, cme_error_t err)
 
 static int dial_number(struct audio_device *device, const char *buf)
 {
-	telephony_dial_number_req(device, &buf[3]);
+	char number[BUF_SIZE];
+	size_t buf_len;
+
+	buf_len = strlen(buf);
+
+	if (buf[buf_len - 1] != ';') {
+		debug("Rejecting non-voice call dial request");
+		return -EINVAL;
+	}
+
+	memset(number, 0, sizeof(number));
+	strncpy(number, &buf[3], buf_len - 4);
+
+	telephony_dial_number_req(device, number);
 
 	return 0;
 }
@@ -897,7 +949,7 @@ static int extended_errors(struct audio_device *device, const char *buf)
 		debug("CME errors disabled for headset %p", hs);
 	}
 
-	return 0;
+	return headset_send(hs, "\r\nOK\r\n");
 }
 
 static int call_waiting_notify(struct audio_device *device, const char *buf)
@@ -915,6 +967,41 @@ static int call_waiting_notify(struct audio_device *device, const char *buf)
 		debug("Call waiting notification disabled for headset %p", hs);
 	}
 
+	return headset_send(hs, "\r\nOK\r\n");
+}
+
+int telephony_operator_selection_rsp(void *telephony_device, cme_error_t err)
+{
+	return telephony_generic_rsp(telephony_device, err);
+}
+
+int telephony_operator_selection_ind(int mode, const char *oper)
+{
+	if (!active_devices)
+		return -ENODEV;
+
+	send_foreach_headset(active_devices, hfp_cmp, "\r\n+COPS:%d,0,%s\r\n",
+				mode, oper);
+	return 0;
+}
+
+static int operator_selection(struct audio_device *device, const char *buf)
+{
+	struct headset *hs = device->headset;
+
+	if (strlen(buf) < 8)
+		return -EINVAL;
+
+	switch (buf[7]) {
+	case '?':
+		telephony_operator_selection_req(device);
+		break;
+	case '=':
+		return headset_send(hs, "\r\nOK\r\n");
+	default:
+		return -EINVAL;
+	}
+
 	return 0;
 }
 
@@ -936,6 +1023,7 @@ static struct event event_callbacks[] = {
 	{ "AT+CLCC", list_current_calls },
 	{ "AT+CMEE", extended_errors },
 	{ "AT+CCWA", call_waiting_notify },
+	{ "AT+COPS", operator_selection },
 	{ 0 }
 };
 
@@ -1377,38 +1465,6 @@ static DBusMessage *hs_connect(DBusConnection *conn, DBusMessage *msg,
 	return NULL;
 }
 
-static int hfp_cmp(struct headset *hs)
-{
-	if (hs->hfp_active)
-		return 0;
-	else
-		return -1;
-}
-
-static void send_foreach_headset(GSList *devices,
-					int (*cmp)(struct headset *hs),
-					char *format, ...)
-{
-	GSList *l;
-	va_list ap;
-
-	for (l = devices; l != NULL; l = l->next) {
-		struct audio_device *device = l->data;
-		struct headset *hs = device->headset;
-		int ret;
-
-		if (cmp && cmp(hs) != 0)
-			continue;
-
-		va_start(ap, format);
-		ret = headset_send_valist(hs, format, ap);
-		if (ret < 0)
-			error("Failed to send to headset: %s (%d)",
-					strerror(-ret), -ret);
-		va_end(ap);
-	}
-}
-
 static int cli_cmp(struct headset *hs)
 {
 	if (!hs->hfp_active)
@@ -1794,8 +1850,8 @@ static GDBusSignalTable headset_signals[] = {
 	{ NULL, NULL }
 };
 
-static void headset_set_channel(struct headset *headset, sdp_record_t *record,
-				uint16_t svc)
+static void headset_set_channel(struct headset *headset,
+				const sdp_record_t *record, uint16_t svc)
 {
 	int ch;
 	sdp_list_t *protos;
@@ -1819,9 +1875,15 @@ static void headset_set_channel(struct headset *headset, sdp_record_t *record,
 		error("Unable to get RFCOMM channel from Headset record");
 }
 
-void headset_update(struct audio_device *dev, sdp_record_t *record, uint16_t svc)
+void headset_update(struct audio_device *dev, uint16_t svc,
+			const char *uuidstr)
 {
 	struct headset *headset = dev->headset;
+	const sdp_record_t *record;
+
+	record = btd_device_get_record(dev->btd_dev, uuidstr);
+	if (!record)
+		return;
 
 	switch (svc) {
 	case HANDSFREE_SVCLASS_ID:
@@ -1883,6 +1945,12 @@ static void headset_free(struct audio_device *dev)
 static void path_unregister(void *data)
 {
 	struct audio_device *dev = data;
+	struct headset *hs = dev->headset;
+
+	if (hs->state > HEADSET_STATE_DISCONNECTED) {
+		debug("Headset unregistered while device was connected!");
+		headset_set_state(dev, HEADSET_STATE_DISCONNECTED);
+	}
 
 	info("Unregistered interface %s on path %s",
 		AUDIO_HEADSET_INTERFACE, dev->path);
@@ -1896,10 +1964,11 @@ void headset_unregister(struct audio_device *dev)
 		AUDIO_HEADSET_INTERFACE);
 }
 
-struct headset *headset_init(struct audio_device *dev, sdp_record_t *record,
-				uint16_t svc)
+struct headset *headset_init(struct audio_device *dev, uint16_t svc,
+				const char *uuidstr)
 {
 	struct headset *hs;
+	const sdp_record_t *record;
 
 	hs = g_new0(struct headset, 1);
 	hs->rfcomm_ch = -1;
@@ -1909,6 +1978,7 @@ struct headset *headset_init(struct audio_device *dev, sdp_record_t *record,
 	hs->hfp_active = FALSE;
 	hs->cli_active = FALSE;
 
+	record = btd_device_get_record(dev->btd_dev, uuidstr);
 	if (!record)
 		goto register_iface;
 
