@@ -1901,7 +1901,7 @@ static DBusMessage *create_device(DBusConnection *conn,
 
 	device_set_temporary(device, FALSE);
 
-	device_browse(device, conn, msg, NULL);
+	device_browse(device, conn, msg, NULL, FALSE);
 
 	adapter->devices = g_slist_append(adapter->devices, device);
 
@@ -2274,7 +2274,6 @@ static void create_stored_device_from_profiles(char *key, char *value,
 	struct btd_adapter *adapter = user_data;
 	GSList *uuids = bt_string2list(value);
 	struct btd_device *device;
-	sdp_list_t *records;
 	bdaddr_t dst;
 	char srcaddr[18], dstaddr[18];
 
@@ -2294,13 +2293,9 @@ static void create_stored_device_from_profiles(char *key, char *value,
 	device_get_address(device, &dst);
 	ba2str(&dst, dstaddr);
 
-	records = read_records(srcaddr, dstaddr);
-
-	device_probe_drivers(device, uuids, records);
-
-	if (records)
-		sdp_list_free(records, (sdp_free_func_t) sdp_record_free);
+	device_probe_drivers(device, uuids);
 
+	g_slist_foreach(uuids, (GFunc) g_free, NULL);
 	g_slist_free(uuids);
 }
 
